//
//  ViewController.m
//  EffectCamera
//
//  Created by 亦野 一郎 on 11/11/22.
//  Copyright (c) 2011年 NUTRACTOR. All rights reserved.
//

#import "ViewController.h"


@implementation ViewController

@synthesize context = _context;

- (void)viewDidLoad
{
    [super viewDidLoad];
    // OpenGLのコンテキスト
    self.context = [[EAGLContext alloc] initWithAPI: kEAGLRenderingAPIOpenGLES2];
    
    if (!self.context) {
        NSLog(@"Failed to create ES context");
    }
    // OpenGLビューを設定
    GLKView *view = (GLKView *)self.view;
    view.context = self.context;
    view.drawableDepthFormat = GLKViewDrawableDepthFormat24;
    // OpenGLのバッファ設定
    glGenRenderbuffers(1, &renderBuffer);
    glBindRenderbuffer(GL_RENDERBUFFER, renderBuffer);
    // OpenGLのコンテキストからCoreImageのコンテキストを生成、コンテキストが共有される
    coreImageContext = [CIContext contextWithEAGLContext: self.context];
    // AVCaptureの設定、beginConfigurationからcommitConfigurationの間
    captureSession = [[AVCaptureSession alloc] init];
    [captureSession beginConfiguration];
    // AVCaptureの品質
    if ([captureSession canSetSessionPreset: AVCaptureSessionPresetMedium]) {
        captureSession.sessionPreset = AVCaptureSessionPresetMedium;
    }
    // 入力デバイスの設定
    AVCaptureDevice *captureDevice = [AVCaptureDevice defaultDeviceWithMediaType: AVMediaTypeVideo];
    NSError *error = nil;
    AVCaptureDeviceInput *input = [AVCaptureDeviceInput deviceInputWithDevice: captureDevice error: &error];
    [captureSession addInput: input];
    // 出力オブジェクトの設定、デーリゲードを設定するとフレーム毎にdidOutputSampleBuffer:メソッドが呼ばれる
    AVCaptureVideoDataOutput *output = [[AVCaptureVideoDataOutput alloc] init];
    [output setAlwaysDiscardsLateVideoFrames: YES]; 
    [output setVideoSettings: [NSDictionary dictionaryWithObject: [NSNumber numberWithInt: kCVPixelFormatType_32BGRA]
                                                          forKey: (id)kCVPixelBufferPixelFormatTypeKey]];    
    [output setSampleBufferDelegate: self queue: dispatch_get_main_queue()];
    [captureSession addOutput: output];
    // AVCaptureの設定終了
    [captureSession commitConfiguration];
    // AVCaptureConnectionの設定
    for (AVCaptureConnection *connection in [output connections]) {
        if ([connection isVideoOrientationSupported]) {
            connection.videoOrientation = AVCaptureVideoOrientationPortrait;
        }
        if ([connection isVideoMinFrameDurationSupported]) {
            connection.videoMinFrameDuration = CMTimeMake(1.0, 15.0);
        }
    }
    // キャプチャセッションをスタート
    [captureSession startRunning];
}

- (void)viewDidUnload
{    
    [super viewDidUnload];
    
    if ([EAGLContext currentContext] == self.context) {
        [EAGLContext setCurrentContext: nil];
    }
	self.context = nil;
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Release any cached data, images, etc. that aren't in use.
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return (interfaceOrientation == UIDeviceOrientationPortrait);
}

#pragma mark - AVCaptureVideoDataOutputSampleBufferDelegate

- (void)captureOutput:(AVCaptureOutput *)captureOutput didOutputSampleBuffer:(CMSampleBufferRef)sampleBuffer fromConnection:(AVCaptureConnection *)connection
{
    CVPixelBufferRef buffer = (CVPixelBufferRef)CMSampleBufferGetImageBuffer(sampleBuffer);
    // AVCaptureDeviceからのバッファをCIImageに変換
    CIImage *ciImage = [CIImage imageWithCVPixelBuffer: buffer];
    // CIFilterでエフェクトをかける
    ciImage = [CIFilter filterWithName: @"CISepiaTone" 
                         keysAndValues: kCIInputImageKey, ciImage, @"inputIntensity", [NSNumber numberWithFloat: 0.8f], nil].outputImage;
    // CoreImageのコンテキストに描画
    CGRect rect = self.view.bounds;
    CGFloat scl = [[UIScreen mainScreen] scale];
    [coreImageContext drawImage: ciImage
                         inRect: CGRectMake(0.0f, 0.0f, rect.size.width * scl, rect.size.height * scl)
                       fromRect: [ciImage extent]];
    // コンテキストを更新すると自動的にOpenGL表示される
    [self.context presentRenderbuffer: GL_RENDERBUFFER];
}

@end
